/*
    This file is part of KJots.

    Copyright (C) 1997 Christoph Neerfeld <Christoph.Neerfeld@home.ivm.de>
                  2002, 2003 Aaron J. Seigo <aseigo@kde.org>
                  2003 Stanislav Kljuhhin <crz@hot.ee>
                  2005-2006 Jaison Lee <lee.jaison@gmail.com>
                  2007-2008 Stephen Kelly <steveire@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "KJotsMain.h"

#include <KActionCollection>
#include <KLocalizedString>
#include <KStandardAction>

#include <QApplication>

#include "KJotsSettings.h"
#include "kjotsbookmarks.h"
#include "kjotsbrowser.h"
#include "kjotsedit.h"
#include "kjotswidget.h"


//----------------------------------------------------------------------
// KJOTSMAIN
//----------------------------------------------------------------------
KJotsMain::KJotsMain(QWidget *parent)
    : KXmlGuiWindow(parent)
    , component(new KJotsWidget(this, this))
{
    KStandardAction::quit(this, &KJotsMain::close, actionCollection());

    setCentralWidget(component);

    setupGUI();
    connect(component, &KJotsWidget::captionChanged, this, qOverload<const QString &>(&KJotsMain::setCaption));
}

bool KJotsMain::queryClose()
{
    return component->queryClose();
}
